==> eopf-2.4.4-py3-none-any.whl: eopf/logging/log.py <==
import logging
from json import load
from logging import CRITICAL, DEBUG, ERROR, INFO, NOTSET, WARNING, Logger, getLogger
from logging.config import dictConfig
from pathlib import Path
from typing import Any, Optional, Union

import yaml
from yaml import SafeLoader

from eopf.config.config import EOConfiguration

EOConfiguration().register_requested_parameter("logging__folder", param_is_optional=True)
EOConfiguration().register_requested_parameter("logging__level", param_is_optional=True)


class EOLogging(object):
    """A factory generating Python Logger based on given configuration"""

    DEFAULT_CFG_DIR = Path(__file__).parent / "conf"
    DEFAULT_LOG_FORMAT = "%(asctime)s - %(name)s : [%(levelname)s] %(module)s : %(funcName)s :: %(message)s"
    DEFAULT_LOG_LEVEL = logging.INFO
    """Default directory containing configurations"""

    _instance: Optional["EOLogging"] = None
    _initialized: bool = False

    def __init__(self) -> None:
        """Initializes by registering logger configurations in the ``cfg_dir``

        Raises
        ----------
        LoggingConfigurationDirDoesNotExist
            When the preset or given logging directory does not exist
        """
        if not EOLogging._initialized:
            self.default_cfg_dir: Optional[Path] = None
            """path to the directory containing logger configurations"""
            self._cfgs: dict[str, Path] = {}
            """dictionary of logger configurations"""
            EOLogging._initialized = True
            self.register_default_conf = False
            self.default_conf_loaded = False

    def __new__(cls) -> "EOLogging":
        """Ensures there is only one object of EOLogFactory (singleton)"""
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def disable_default_conf(self) -> None:
        """
        Disable the loading of the default conf in the cfg_dir folder
        Returns
        -------

        """
        self.register_default_conf = False

    def enable_default_conf(self) -> None:
        """
        Enable the loading of the default conf in the cfg_dir folder
        Returns
        -------

        """
        self.register_default_conf = True
        self.default_conf_loaded = False

    @property
    def config_dict(self) -> dict[str, Path]:
        """

        Returns the dictionary of loaded files
        -------

        """
        return self._cfgs

    @staticmethod
    def setup_basic_config(
        level: Optional[int] = None,
        format: Optional[str] = None,
        file: Optional[str] = None,
    ) -> None:
        from eopf.exceptions.warnings import LoggingLevelIsNoneStandard

        if format is None:
            format_str = EOLogging.DEFAULT_LOG_FORMAT
        else:
            format_str = format
        # override the logging level from the cfg file
        if level and level != NOTSET:
            if level not in [DEBUG, INFO, WARNING, ERROR, CRITICAL]:
                raise LoggingLevelIsNoneStandard(
                    "The given log level is register_requested_parameter to a value which is none Python standard",
                )
        elif EOConfiguration().has_value("logging__level"):
            level = logging.getLevelName(EOConfiguration().logging__level)
            if level not in [DEBUG, INFO, WARNING, ERROR, CRITICAL]:
                raise LoggingLevelIsNoneStandard(
                    "The given log level is register_requested_parameter to a value which is none Python standard",
                )
        else:
            level = EOLogging.DEFAULT_LOG_LEVEL
        if file is not None:
            logging.basicConfig(level=level, format=format_str, filename=file)
        else:
            logging.basicConfig(level=level, format=format_str)

    def reset(self) -> None:
        """
        Clear all the logging configuration
        Beware : if someone still has a reference to the logger it  will not be destroyed by the garbage collect
        but will no longer have handlers
        Returns
        -------

        """
        self.default_conf_loaded = False
        self._cfgs = {}
        # Iterate through all loggers and their handlers
        for logger_name, logger in logging.Logger.manager.loggerDict.items():
            if isinstance(logger, logging.Logger):
                # Close and remove handlers associated with the logger
                for handler in logger.handlers:
                    handler.close()
                    logger.removeHandler(handler)

        # Clear the logger hierarchy by removing all loggers from the logger dictionary
        logging.Logger.manager.loggerDict.clear()

    def set_default_cfg_dir(self, url: Optional[Union[str, Path]]) -> None:
        """Set the ``cfg_dir`` parameter, remove old configurations and add those in the new ``cfg_dir``

        Parameters
        ----------
        url: Union[str, Path]
            path to a directory containing logger configurations

        Raises
        ----------
        LoggingConfigurationDirDoesNotExist
            When the preset or given logging directory does not exist
        """
        from eopf.exceptions import LoggingConfigurationDirDoesNotExist

        if url is None:
            """Set the default config dir according to the eopf configuration folder if none given"""
            self.default_cfg_dir = self.get_default_cfg_dir()
        elif not isinstance(url, Path):
            self.default_cfg_dir = Path(url)
        else:
            self.default_cfg_dir = url
        if not self.default_cfg_dir.is_dir():
            raise LoggingConfigurationDirDoesNotExist("The logging configuration directory must exist")
        self.enable_default_conf()
        self.register_cfg_dir(self.default_cfg_dir, True)

    def get_default_cfg_dir(self) -> Path:
        if self.default_cfg_dir is None:
            conf = EOConfiguration()
            if conf.has_value("logging__folder"):
                self.default_cfg_dir = Path(conf.logging__folder)
            else:
                self.default_cfg_dir = self.DEFAULT_CFG_DIR
        return self.default_cfg_dir

    def register_cfg(self, cfg_name: str, cfg_path: Union[Path, str]) -> None:
        """Register a logger configuration by name and path

        Parameters
        ----------
        cfg_name: str
            name of the logger configuration

        cfg_path: Union[Path, str]
            path of the logger configuration

        Raises
        ----------
        FileNotFoundError
            When a file is not found at given location
        LoggingConfigurationFileTypeNotSupported
            When the logging file name does not have a .json extension
        """
        from eopf.exceptions import (
            LoggingConfigurationFileTypeNotSupported,
            LoggingDictConfigurationInvalid,
        )

        # if the configuration does not exists register it
        if cfg_name not in self._cfgs:
            if not isinstance(cfg_path, Path):
                cfg_file_path = Path(cfg_path)
            else:
                cfg_file_path = cfg_path
            if not cfg_file_path.is_file():
                raise FileNotFoundError(f"File {cfg_file_path} can not be found")
            if cfg_file_path.suffix not in [".json", ".yaml"]:
                raise LoggingConfigurationFileTypeNotSupported("Unsupported configuration file type")
            if cfg_file_path.suffix == ".json":
                # load_file the json configuration
                try:
                    logging.debug(f"Registering logging config for {cfg_name} with file {cfg_file_path}")
                    with open(cfg_file_path, "r") as f:
                        json_cfg = load(f)
                        EOLogging._dict_cfg(json_cfg)
                        self._cfgs[cfg_name] = cfg_file_path
                except Exception as e:
                    raise LoggingDictConfigurationInvalid(
                        f"Can not setup dict given logging configuration due to: {e} in file {cfg_file_path}",
                    )
            elif cfg_file_path.suffix == ".yaml":
                # load_file the json configuration
                try:
                    logging.debug(f"Registering logging config for {cfg_name} with file {cfg_file_path}")
                    with open(cfg_file_path, "r") as f:
                        yaml_cfg = yaml.load(f, Loader=SafeLoader)
                        EOLogging._dict_cfg(yaml_cfg)
                        self._cfgs[cfg_name] = cfg_file_path
                except Exception as e:
                    raise LoggingDictConfigurationInvalid(
                        f"Can not setup dict given logging configuration due to: {e} in file {cfg_file_path}",
                    )

    def register_cfg_dir(self, cfg_dir: Path, mandatory: bool = False) -> None:
        """

        Parameters
        ----------
        cfg_dir : directory to load_file from
        mandatory : flag if the dir should mandatory have conf file in it

        Returns
        -------
        None
        """
        from eopf.exceptions.warnings import NoLoggingConfigurationFile

        # register the configurations in the cfg_dir
        configuration_present = False
        if cfg_dir is not None:
            # root logger prevails
            for cfg_path in cfg_dir.glob("root*.json"):
                configuration_present = True
                self.register_cfg(cfg_path.stem, cfg_path)
            for cfg_path in cfg_dir.glob("*.json"):
                configuration_present = True
                self.register_cfg(cfg_path.stem, cfg_path)
            for cfg_path in cfg_dir.glob("*.yaml"):
                configuration_present = True
                self.register_cfg(cfg_path.stem, cfg_path)

        if not configuration_present and mandatory:
            raise NoLoggingConfigurationFile(
                f"No logging configuration file .json/.yaml is present in {self.default_cfg_dir}",
            )

    @staticmethod
    def _dict_cfg(cfg: dict[Any, Any]) -> None:
        """Setting a logger configuration by providing a config dict

        Parameters
        ----------
        cfg: dict
            dictionary containing the logger configuration

        Raises
        ----------
        LoggingDictConfigurationIsNotValid
            When the provided configuration is not valid

        """

        from eopf.exceptions.warnings import LoggingDictConfigurationIsNotValid

        try:
            dictConfig(cfg)
        except Exception as e:
            raise LoggingDictConfigurationIsNotValid(f"Invalid configuration, reason: {e}")

    def get_cfg_path(self, cfg_name: str) -> Path:
        """Retrieve a logger configuration path by its name

        Parameters
        ----------
        cfg_name: str
            name of the logger configuration

        Raises
        ----------
        LoggingConfigurationNotRegistered
            When a given logging configuration name is not registered
        """
        from eopf.exceptions import LoggingConfigurationNotRegistered

        if cfg_name not in self._cfgs:
            raise LoggingConfigurationNotRegistered(f"No log configuration {cfg_name} is registered")
        return self._cfgs[cfg_name]

    def get_logger(
        self,
        name: str = "eopf",
        level: Optional[int] = None,
    ) -> Logger:
        """Retrieve a logger by specifying the name of the configuration
        and register_requested_parameter the logger's level
        Warning : this will register_requested_parameter the level for all code having this logger

        Parameters
        ----------
        name: str
            name of the logger or actual configuration dictionary

        level: int
            logger level

        Raises
        ----------
        LoggingConfigurationFileIsNotValid
            When a given logging configuration file .conf/.yaml cannot be applied
        """
        from eopf.exceptions.warnings import LoggingLevelIsNoneStandard

        # if not init try to init with defaults
        if not self.default_conf_loaded and self.register_default_conf:
            self.set_default_cfg_dir(url=None)
        # if the requested logger is defined in the config file it will be returned else a new one will be created with
        # a default config
        logger = getLogger(name)
        # override the logging level from the cfg file
        if level and level != NOTSET:
            if level not in [DEBUG, INFO, WARNING, ERROR, CRITICAL]:
                raise LoggingLevelIsNoneStandard(
                    "The given log level is register_requested_parameter to a value which is none Python standard",
                )
            logger.setLevel(level=level)
        elif EOConfiguration().has_value("logging__level"):
            level_str: str = str(EOConfiguration().logging__level).upper()
            if level_str not in ["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]:
                raise LoggingLevelIsNoneStandard(
                    "The given log level is register_requested_parameter to a value which is none Python standard",
                )
            logger.setLevel(level_str)

        logging.debug(f"Logger {name} initialised with level {logger.getEffectiveLevel()}")
        return logger

==> eopf-2.5.5-py3-none-any.whl: eopf/logging/log.py <==
import logging
from json import load
from logging import CRITICAL, DEBUG, ERROR, INFO, NOTSET, WARNING, Logger, getLogger
from logging.config import dictConfig
from pathlib import Path
from typing import Any, Optional, Union

import yaml
from yaml import SafeLoader

from eopf.config.config import EOConfiguration

DEFAULT_CFG_DIR = Path(__file__).parent / "conf"

EOConfiguration().register_requested_parameter(
    "logging__folder",
    param_is_optional=True,
    default_value=DEFAULT_CFG_DIR,
    description="Folder to look for json/yaml logging dictConfig files ",
)
EOConfiguration().register_requested_parameter(
    "logging__level",
    param_is_optional=True,
    default_value="INFO",
    description="Default logging level",
)


class EOLogging(object):
    """A factory generating Python Logger based on given configuration"""

    DEFAULT_LOG_FORMAT = "%(asctime)s - %(name)s : [%(levelname)s] %(module)s : %(funcName)s :: %(message)s"
    DEFAULT_LOG_LEVEL = logging.INFO
    """Default directory containing configurations"""

    _instance: Optional["EOLogging"] = None
    _initialized: bool = False

    def __init__(self) -> None:
        """Initializes by registering logger configurations in the ``cfg_dir``

        Raises
        ----------
        LoggingConfigurationDirDoesNotExist
            When the preset or given logging directory does not exist
        """
        if not EOLogging._initialized:
            self.default_cfg_dir: Optional[Path] = None
            """path to the directory containing logger configurations"""
            self._cfgs: dict[str, Path] = {}
            """dictionary of logger configurations"""
            EOLogging._initialized = True
            self.register_default_conf = False
            self.default_conf_loaded = False

    def __new__(cls) -> "EOLogging":
        """Ensures there is only one object of EOLogFactory (singleton)"""
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def disable_default_conf(self) -> None:
        """
        Disable the loading of the default conf in the cfg_dir folder
        Returns
        -------

        """
        self.register_default_conf = False

    def enable_default_conf(self) -> None:
        """
        Enable the loading of the default conf in the cfg_dir folder
        Returns
        -------

        """
        self.register_default_conf = True
        self.default_conf_loaded = False

    @property
    def config_dict(self) -> dict[str, Path]:
        """

        Returns the dictionary of loaded files
        -------

        """
        return self._cfgs

    @staticmethod
    def setup_basic_config(
        level: Optional[int] = None,
        format: Optional[str] = None,
        file: Optional[str] = None,
    ) -> None:
        from eopf.exceptions.warnings import LoggingLevelIsNoneStandard

        if format is None:
            format_str = EOLogging.DEFAULT_LOG_FORMAT
        else:
            format_str = format
        # override the logging level from the cfg file
        if level and level != NOTSET:
            if level not in [DEBUG, INFO, WARNING, ERROR, CRITICAL]:
                raise LoggingLevelIsNoneStandard(
                    "The given log level is register_requested_parameter to a value which is none Python standard",
                )
        elif EOConfiguration().has_value("logging__level"):
            level = logging.getLevelName(EOConfiguration().logging__level)
            if level not in [DEBUG, INFO, WARNING, ERROR, CRITICAL]:
                raise LoggingLevelIsNoneStandard(
                    "The given log level is register_requested_parameter to a value which is none Python standard",
                )
        else:
            level = EOLogging.DEFAULT_LOG_LEVEL
        if file is not None:
            logging.basicConfig(level=level, format=format_str, filename=file)
        else:
            logging.basicConfig(level=level, format=format_str)

    def reset(self) -> None:
        """
        Clear all the logging configuration
        Beware : if someone still has a reference to the logger it  will not be destroyed by the garbage collect
        but will no longer have handlers
        Returns
        -------

        """
        self.default_conf_loaded = False
        self._cfgs = {}
        # Iterate through all loggers and their handlers
        for logger_name, logger in logging.Logger.manager.loggerDict.items():
            if isinstance(logger, logging.Logger):
                # Close and remove handlers associated with the logger
                for handler in logger.handlers:
                    handler.close()
                    logger.removeHandler(handler)

        # Clear the logger hierarchy by removing all loggers from the logger dictionary
        logging.Logger.manager.loggerDict.clear()

    def set_default_cfg_dir(self, url: Optional[Union[str, Path]]) -> None:
        """Set the ``cfg_dir`` parameter, remove old configurations and add those in the new ``cfg_dir``

        Parameters
        ----------
        url: Union[str, Path]
            path to a directory containing logger configurations

        Raises
        ----------
        LoggingConfigurationDirDoesNotExist
            When the preset or given logging directory does not exist
        """
        from eopf.exceptions import LoggingConfigurationDirDoesNotExist

        if url is None:
            """Set the default config dir according to the eopf configuration folder if none given"""
            self.default_cfg_dir = self.get_default_cfg_dir()
        elif not isinstance(url, Path):
            self.default_cfg_dir = Path(url)
        else:
            self.default_cfg_dir = url
        if not self.default_cfg_dir.is_dir():
            raise LoggingConfigurationDirDoesNotExist("The logging configuration directory must exist")
        self.enable_default_conf()
        self.register_cfg_dir(self.default_cfg_dir, True)

    def get_default_cfg_dir(self) -> Path:
        if self.default_cfg_dir is None:
            conf = EOConfiguration()
            if conf.has_value("logging__folder"):
                self.default_cfg_dir = Path(conf.logging__folder)
            else:
                self.default_cfg_dir = DEFAULT_CFG_DIR
        return self.default_cfg_dir

    def register_cfg(self, cfg_name: str, cfg_path: Union[Path, str]) -> None:
        """Register a logger configuration by name and path

        Parameters
        ----------
        cfg_name: str
            name of the logger configuration

        cfg_path: Union[Path, str]
            path of the logger configuration

        Raises
        ----------
        FileNotFoundError
            When a file is not found at given location
        LoggingConfigurationFileTypeNotSupported
            When the logging file name does not have a .json extension
        """
        from eopf.exceptions import (
            LoggingConfigurationFileTypeNotSupported,
            LoggingDictConfigurationInvalid,
        )

        # if the configuration does not exists register it
        if cfg_name not in self._cfgs:
            if not isinstance(cfg_path, Path):
                cfg_file_path = Path(cfg_path)
            else:
                cfg_file_path = cfg_path
            if not cfg_file_path.is_file():
                raise FileNotFoundError(f"File {cfg_file_path} can not be found")
            if cfg_file_path.suffix not in [".json", ".yaml"]:
                raise LoggingConfigurationFileTypeNotSupported("Unsupported configuration file type")
            if cfg_file_path.suffix == ".json":
                # load_file the json configuration
                try:
                    logging.debug(f"Registering logging config for {cfg_name} with file {cfg_file_path}")
                    with open(cfg_file_path, "r") as f:
                        json_cfg = load(f)
                        EOLogging._dict_cfg(json_cfg)
                        self._cfgs[cfg_name] = cfg_file_path
                except Exception as e:
                    raise LoggingDictConfigurationInvalid(
                        f"Can not setup dict given logging configuration due to: {e} in file {cfg_file_path}",
                    )
            elif cfg_file_path.suffix == ".yaml":
                # load_file the json configuration
                try:
                    logging.debug(f"Registering logging config for {cfg_name} with file {cfg_file_path}")
                    with open(cfg_file_path, "r") as f:
                        yaml_cfg = yaml.load(f, Loader=SafeLoader)
                        EOLogging._dict_cfg(yaml_cfg)
                        self._cfgs[cfg_name] = cfg_file_path
                except Exception as e:
                    raise LoggingDictConfigurationInvalid(
                        f"Can not setup dict given logging configuration due to: {e} in file {cfg_file_path}",
                    )

    def register_cfg_dir(self, cfg_dir: Path, mandatory: bool = False) -> None:
        """

        Parameters
        ----------
        cfg_dir : directory to load_file from
        mandatory : flag if the dir should mandatory have conf file in it

        Returns
        -------
        None
        """
        from eopf.exceptions.warnings import NoLoggingConfigurationFile

        # register the configurations in the cfg_dir
        configuration_present = False
        if cfg_dir is not None:
            # root logger prevails
            for cfg_path in cfg_dir.glob("root*.json"):
                configuration_present = True
                self.register_cfg(cfg_path.stem, cfg_path)
            for cfg_path in cfg_dir.glob("*.json"):
                configuration_present = True
                self.register_cfg(cfg_path.stem, cfg_path)
            for cfg_path in cfg_dir.glob("*.yaml"):
                configuration_present = True
                self.register_cfg(cfg_path.stem, cfg_path)

        if not configuration_present and mandatory:
            raise NoLoggingConfigurationFile(
                f"No logging configuration file .json/.yaml is present in {self.default_cfg_dir}",
            )

    @staticmethod
    def _dict_cfg(cfg: dict[Any, Any]) -> None:
        """Setting a logger configuration by providing a config dict

        Parameters
        ----------
        cfg: dict
            dictionary containing the logger configuration

        Raises
        ----------
        LoggingDictConfigurationIsNotValid
            When the provided configuration is not valid

        """

        from eopf.exceptions.warnings import LoggingDictConfigurationIsNotValid

        try:
            dictConfig(cfg)
        except Exception as e:
            raise LoggingDictConfigurationIsNotValid(f"Invalid configuration, reason: {e}")

    def get_cfg_path(self, cfg_name: str) -> Path:
        """Retrieve a logger configuration path by its name

        Parameters
        ----------
        cfg_name: str
            name of the logger configuration

        Raises
        ----------
        LoggingConfigurationNotRegistered
            When a given logging configuration name is not registered
        """
        from eopf.exceptions import LoggingConfigurationNotRegistered

        if cfg_name not in self._cfgs:
            raise LoggingConfigurationNotRegistered(f"No log configuration {cfg_name} is registered")
        return self._cfgs[cfg_name]

    def get_logger(
        self,
        name: str = "eopf",
        level: Optional[int] = None,
    ) -> Logger:
        """Retrieve a logger by specifying the name of the configuration
        and register_requested_parameter the logger's level
        Warning : this will register_requested_parameter the level for all code having this logger

        Parameters
        ----------
        name: str
            name of the logger or actual configuration dictionary

        level: int
            logger level

        Raises
        ----------
        LoggingConfigurationFileIsNotValid
            When a given logging configuration file .conf/.yaml cannot be applied
        """
        from eopf.exceptions.warnings import LoggingLevelIsNoneStandard

        # if not init try to init with defaults
        if not self.default_conf_loaded and self.register_default_conf:
            self.set_default_cfg_dir(url=None)
        # if the requested logger is defined in the config file it will be returned else a new one will be created with
        # a default config
        logger = getLogger(name)
        # override the logging level from the cfg file
        if level and level != NOTSET:
            if level not in [DEBUG, INFO, WARNING, ERROR, CRITICAL]:
                raise LoggingLevelIsNoneStandard(
                    "The given log level is register_requested_parameter to a value which is none Python standard",
                )
            logger.setLevel(level=level)
        elif EOConfiguration().has_value("logging__level"):
            level_str: str = str(EOConfiguration().logging__level).upper()
            if level_str not in ["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]:
                raise LoggingLevelIsNoneStandard(
                    "The given log level is register_requested_parameter to a value which is none Python standard",
                )
            logger.setLevel(level_str)

        logging.debug(f"Logger {name} initialised with level {logger.getEffectiveLevel()}")
        return logger
